using MorphoNet;
using System;
using System.Collections;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using UnityEngine;
using UnityEngine.Networking;

namespace MorphoNet
{
    public class SocketCommunication
    {
        //request parameters
        private readonly static int BUF_SIZE = 1024;
        private readonly string _IpString;
        private readonly ushort _Port;

        //return parameters
        private byte[] _Command;
        private bool _Error;
        private WWWForm _Form;

        public SocketCommunication(string ipString, ushort port)
        {
            _IpString = ipString;
            _Port = port;
            Debug.LogWarning("init");
        }

        public SocketCommunication(string ipString, int port)
        {
            _IpString = ipString;
            _Port = (ushort)port;
            Debug.LogWarning("init");
        }

        public byte[] GetCommand() => _Command;

        public bool IsError() => _Error;

        public IEnumerator Receive()
        {
            Debug.LogWarning("recv");
            _Command = null;
            _Error = false;

            Thread SocketThread = new Thread(ReceiveRequestThread);
            SocketThread.IsBackground = true;
            SocketThread.Start();

            yield return new WaitUntil(() => _Command != null || _Error == true);
        }

        private void ReceiveRequestThread()
        {
            TcpListener server = null;
            try
            {
                IPAddress localAddr = IPAddress.Parse(_IpString);
                server = new TcpListener(localAddr, _Port);
                TcpClient client;
                NetworkStream stream;
                server.Start();

                byte[] buffer = new byte[BUF_SIZE];

                MorphoDebug.Log("Waiting for connection...");
                client = server.AcceptTcpClient();
                MorphoDebug.Log("Connected!");

                stream = client.GetStream();
                byte[] message = null;
                int i;
                int index = 0;

                while ((i = stream.Read(buffer, 0, BUF_SIZE)) != 0)
                {
                    if (message != null)
                    {
                        //add bytes from stream to final message
                        for (int j = 0; j < i; j++)
                            message[j + index] = buffer[j];
                        index += i;
                    }
                    else
                    {
                        //first buffer reading: find the header with the message size
                        string data = Encoding.UTF8.GetString(buffer, 0, i);
                        if (!data.Contains(";")) // if no header detected
                        {
                            MorphoDebug.LogError("ERROR: could not receive header of request.");
                        }
                        else
                        {
                            //read message size
                            string head = data.Split(";")[0];
                            if(int.TryParse(head,out int size))
                            {
                                message = new byte[size];
                                //then add the message minus the header, and increment index properly (+1 to account for the ; separator)
                                for (int j = head.Length+1; j < i; j++)
                                    message[j-(head.Length+1)] = buffer[j];
                                index += i - (head.Length+1);
                            }
                            else
                            {
                                MorphoDebug.LogError($"ERROR:header could not be parsed to int: {head}");
                                return;
                            }
                        }
                    }
                    
                }
                _Command = message;
                client.Close();
            }
            catch (SocketException e)
            {
                MorphoDebug.LogError("SocketException: " + e);
                MorphoDebug.LogError(e.StackTrace);
                _Error = true;
                _Command = Encoding.UTF8.GetBytes("ERROR");
                //more exception management ?
            }
            finally
            {
                if (server != null)
                    server.Stop();
            }
        }

        public IEnumerator Send(WWWForm data)
        {
            Debug.LogWarning("send");
            _Command = null;
            _Form = data;
            _Error = false;
            Thread SocketThread = new Thread(SendRequestThread);
            SocketThread.IsBackground = true;
            SocketThread.Start();

            yield return new WaitUntil(() => _Command != null || _Error == true);
        }

        private void SendRequestThread()
        {
            Socket soc = null;
            int bytesSent = 0;
            try
            {
                soc = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                Debug.Log(_IpString);
                IPAddress ipAdd = IPAddress.Parse(_IpString);
                IPEndPoint remoteEP = new IPEndPoint(ipAdd, _Port);
                soc.Connect(remoteEP);
                //get len of message before sening header for metter memory handling
                byte[] msgData = _Form.data;
                int msgSize = msgData.Length;
                byte[] headData = Encoding.UTF8.GetBytes($"{msgSize};");

                //ad 1 sec timout
                soc.SendTimeout = 1000;
                bytesSent += soc.Send(headData);
                bytesSent += soc.Send(msgData);
                float pct = ((float)(bytesSent - headData.Length) / (float)msgSize) * 100.0f;
                _Command = Encoding.UTF8.GetBytes($"{pct}% message sent");
            }
            catch (SocketException e)
            {
                MorphoDebug.LogError("SocketException: " + e);
                MorphoDebug.LogError(e.StackTrace);
                _Error = true;
                _Command = Encoding.UTF8.GetBytes($"ERROR: bytes sent {bytesSent}");
                //more exception management ?
            }
            finally
            {
                if (soc != null)
                    soc.Close();
            }
        }
    }
}
