using System;
using System.Collections;
using System.Collections.Generic;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using UnityEngine;

public class SocketManager
{

    //request parameters
    private readonly static int BUF_SIZE = 1024;
    private readonly string _IpString;
    private readonly ushort _Port;

    private Thread _SocketThread;

    //return parameters
    private Queue<byte[]> _MessageQueue = new Queue<byte[]>();

    private Queue<WWWForm> _SendQueue = new Queue<WWWForm>();

    private bool _SendError;
    private bool _MessageSent = false;
    private float _SendPct;

    public SocketManager(string ipString, ushort port)
    {
        _MessageQueue = new Queue<byte[]>();
        _IpString = ipString;
        _Port = port;

    }

    public SocketManager(string ipString, int port)
    {
        _MessageQueue = new Queue<byte[]>();
        _IpString = ipString;
        _Port = (ushort)port;

    }

    public Queue<byte[]> GetMessages() => _MessageQueue;

    public bool IsMessageSent() => _MessageSent;
    public float GetSendPercent() => _SendPct;
    public bool IsError() => _SendError;

    public void StartReceiving()
    {
        //Thread SocketThread = new Thread(ReceiveRequestThread);
        _SocketThread = new Thread(RecvLoop);
        _SocketThread.IsBackground = true;
        _SocketThread.Start();
    }

    public void StartSending()
    {
        _SocketThread = new Thread(SendLoop);
        _SocketThread.IsBackground = true;
        _SocketThread.Start();
    }

    public void EndThread()
    {
        if (_SocketThread != null)
        {
            _SocketThread.Abort();
        }
    }

    private void RecvLoop()
    {
        TcpListener server = null;
        try
        {
            IPAddress localAddr = IPAddress.Parse(_IpString);
            server = new TcpListener(localAddr, _Port);
            TcpClient client;
            NetworkStream stream;
            server.Start();

            byte[] buffer = new byte[BUF_SIZE];
            while (true)
            {
                Debug.Log("Waiting for connection...");
                client = server.AcceptTcpClient();
                Debug.Log("Connected!");
                //DateTime time = DateTime.Now;

                stream = client.GetStream();
                byte[] message = null;
                byte[] temp_message = new byte[0];
                int i;
                int index = 0;
                int message_size = -1;
                while ((i = stream.Read(buffer, 0, BUF_SIZE)) != 0)//remove command null check for later
                {
                    byte[] prepared_buffer = buffer;
                    //first of all, setup if message is null:
                    if (message == null)
                    {
                        //if there is a temp buffer (bits of incomplete header), read it and append the buffer
                        index = 0;
                        byte[] temp_message_2 = new byte[i + temp_message.Length];
                        //first buffer reading: find the header with the message size
                        for (int j = 0; j < temp_message.Length; j++)
                        {
                            temp_message_2[index] = temp_message[j];
                            index++;
                        }
                        for (int j = 0; j < i; j++)
                        {
                            temp_message_2[index] = buffer[j];
                            index++;
                        }
                        //check the header and get the message size
                        string data = Encoding.UTF8.GetString(temp_message_2, 0, temp_message_2.Length);
                        if (!data.Contains(";")) // if no header detected
                        {
                            // in this case there is and error: the header HAS to be read
                            Debug.LogError("ERROR: could not receive header of request.");
                            return;
                        }
                        else
                        {
                            string head = data.Split(";")[0];
                            if (int.TryParse(head, out int size))
                            {
                                message_size = size;//get message size
                                message = new byte[size];
                                index = 0; //reset index

                                //then remove the header from the buffer to read (head.Length + 1) to account for the ;
                                prepared_buffer = new byte[temp_message_2.Length - (head.Length + 1)];
                                i = i + temp_message.Length - (head.Length + 1); //actually update size of buffer read i with bytes before
                                Buffer.BlockCopy(temp_message_2, (head.Length + 1), prepared_buffer, 0, prepared_buffer.Length);
                            }
                            else //if we can't read the header here, there is an error!
                            {
                                Debug.LogError($"ERROR:header could not be parsed to int: {head}");
                                return;
                            }

                        }
                    }
                    
                    //then read the buffer (or just start reading it if message was not null!
                    if (message != null)//if message is already initialized
                    {
                        for (int j = 0; j < i; j++)// append the buffer to the message
                        {
                            message[index] = prepared_buffer[j];
                            index++;
                            if (index == message.Length) // if we reach the end of the message (not buffer, message)
                            {
                                _MessageQueue.Enqueue(message);//add the message to the reception queue
                                index = 0;
                                if (j < i)//if there is still data remaining in the buffer
                                {
                                    int overall_index = j + 1;
                                    while (true)//for as long as possible (as long as there is still data in the buffer
                                    {
                                        byte[] buf = new byte[i - overall_index];
                                        Buffer.BlockCopy(prepared_buffer, overall_index, buf, 0, buf.Length);
                                        string data = Encoding.UTF8.GetString(buf, 0, buf.Length);
                                        if (!data.Contains(";")) // if no header detected
                                        {
                                            temp_message = buf;//then the remainging buffer data will be treated in the next iteration (since it means that the header is not full
                                            //set index value & message ?
                                            message = null;
                                            message_size = -1;
                                            break;
                                        }
                                        else //if there is a ; (meaning header)
                                        {
                                            //read message size
                                            string head = data.Split(";")[0];
                                            if (int.TryParse(head, out int size))
                                            {
                                                message_size = size;//get message size
                                                message = new byte[size];
                                                index = 0; //reset index
                                                if (buf.Length - (head.Length + 1) < message.Length)// if the message is longer that the remaining buffer
                                                {
                                                    for(int k = (head.Length + 1); k < buf.Length; k++)//fill the messsage with the whole remaining buffer
                                                    {
                                                        message[index] = buf[k];
                                                        index++;
                                                    }
                                                    break;// break the buffer loop: we've finished reading it. Onto the next !
                                                }
                                                else//else if the whole message is in the buffer
                                                {
                                                    for (int k = (head.Length + 1); k < message.Length + (head.Length + 1); k++)//fill the messsage (starting after the header)
                                                    {
                                                        message[index] = buf[k];
                                                        index++;
                                                        
                                                    }
                                                    _MessageQueue.Enqueue(message);//add the message to the reception queue, since it is finished
                                                    overall_index += index + (head.Length + 1); //augment the overall index to fetch the next part of the buffer in the next iteration
                                                                            //then at this point we'll go back to the beginnig of the while(true) loop to check in the buffer
                                                }
                                            }
                                            else//if there was a ; but we couldn't read it, there is an error.
                                            {
                                                Debug.LogError($"ERROR:header could not be parsed to int: {head}");
                                                return;
                                            }
                                        }

                                    }
                                }
                                else //if no more data in the buffer
                                {
                                    message_size = -1;
                                    index = 0;
                                    message = null;
                                }

                                break; //always break here: whatever happens we will stop iterating over the initial buffer
                            }
                        }
                    }
                    buffer = new byte[BUF_SIZE];
                }
                Debug.LogError("closing the connection");
                client.Close();
            }

        }
        catch (SocketException e)
        {
            Debug.LogError("SocketException: " + e);
            Debug.LogError(e.StackTrace);
            //more exception management ?
        }
        catch (ThreadAbortException e)
        {
            Debug.LogWarning("thread aborted");
            //more exception management ?
        }
        finally
        {
            Debug.LogError("finally ???");
            if (server != null)
                server.Stop();
        }
    }

    public void AddMessage(WWWForm form)
    {
        _SendQueue.Enqueue(form);
    }

    public void SendLoop()
    {
        Socket soc = null;
        int bytesSent = 0;
        try
        {
            soc = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            Debug.Log(_IpString);
            IPAddress ipAdd = IPAddress.Parse(_IpString);
            IPEndPoint remoteEP = new IPEndPoint(ipAdd, _Port);
            soc.Connect(remoteEP);

            while (true)
            {
                if (_SendQueue.Count > 0)
                {
                    
                    _MessageSent = false;
                    _SendPct = 0;
                    _SendError = false;
                    WWWForm form = _SendQueue.Dequeue();
                    byte[] msgData = form.data;
                    int msgSize = msgData.Length;
                    byte[] headData = Encoding.UTF8.GetBytes($"{msgSize};");
                    Debug.Log("sending... " +Encoding.UTF8.GetString(msgData));
                    bytesSent = 0;
                    //get len of message before sening header for metter memory handling
                    bytesSent += soc.Send(headData);
                    bytesSent += soc.Send(msgData);
                    _SendPct = ((float)(bytesSent - headData.Length) / (float)msgSize) * 100.0f;
                    Debug.Log($"{_SendPct}% message sent");
                    if (_SendPct != 100)
                        _SendError = true;
                    _MessageSent = true;
                    Debug.LogWarning("finished sending!");
                }
            }



        }
        catch (SocketException e)
        {
            Debug.LogError("SocketException: " + e);
            Debug.LogError(e.StackTrace);
            //more exception management ?
        }
        catch (ThreadAbortException e)
        {
            Debug.LogWarning("thread aborted");
            //more exception management ?
        }
        finally
        {
            if (soc != null)
                soc.Close();
        }
    }

    public static string ByteArrayToString(byte[] ba)
    {
        StringBuilder hex = new StringBuilder(ba.Length * 2);
        foreach (byte b in ba)
            hex.AppendFormat("{0:x2}", b);
        return hex.ToString();
    }

    public byte[] ConcatenateByteArrays(byte[] b1, byte[] b2)
    {
        byte[] ret = new byte[b1.Length + b2.Length];
        //first buffer reading: find the header with the message size
        int index = 0;
        for (int j = 0; j < b1.Length; j++)
        {
            ret[j] = b1[j];
            index++;
        }
        for (int j = 0; j < b2.Length; j++)
        {
            ret[index] = b2[j];
            index++;
        }
        return ret;
    }
}
